@@ -47,6 +47,7 @@ class TestBasic(DRYTest):
     def test_bools_and_or(self):
         self.t('True and False', False)
         self.t('True or False', True)
+        self.t('False or False', False)
         self.t('1 - 1 or 21', 21)
         self.t('1 - 1 and 11', 0)
         self.t('110 == 100 + 10 and True', True)
@@ -139,6 +140,19 @@ class TestBasic(DRYTest):
         self.t('"I" not in "team"', True)
         self.t('"U" in "RUBBISH"', True)
 
+    def test_is(self):
+        self.t('1 is 1', True)
+        self.t('1 is 2', False)
+        self.t('1 is "a"', False)
+        self.t('1 is None', False)
+        self.t('None is None', True)
+
+        self.t('1 is not 1', False)
+        self.t('1 is not 2', True)
+        self.t('1 is not "a"', True)
+        self.t('1 is not None', True)
+        self.t('None is not None', False)
+
 
 class TestFunctions(DRYTest):
     """ Functions for expressions to play with """
@@ -148,7 +162,7 @@ class TestFunctions(DRYTest):
 
         # write to the file:
 
-        with open("file.txt", 'w') as f:
+        with open("testfile.txt", 'w') as f:
             f.write("42")
 
         # define the function we'll send to the eval'er
@@ -161,12 +175,12 @@ class TestFunctions(DRYTest):
         # simple load:
 
         self.s.functions = {"read": load_file}
-        self.t("read('file.txt')", "42")
+        self.t("read('testfile.txt')", "42")
 
         # and we should have *replaced* the default functions. Let's check:
 
         with self.assertRaises(simpleeval.FunctionNotDefined):
-            self.t("int(read('file.txt'))", 42)
+            self.t("int(read('testfile.txt'))", 42)
 
         # OK, so we can load in the default functions as well...
 
@@ -174,7 +188,7 @@ class TestFunctions(DRYTest):
 
         # now it works:
 
-        self.t("int(read('file.txt'))", 42)
+        self.t("int(read('testfile.txt'))", 42)
 
     def test_randoms(self):
         """ test the rand() and randint() functions """
@@ -191,7 +205,10 @@ class TestFunctions(DRYTest):
 
     def test_methods(self):
         self.t('"WORD".lower()', 'word')
+        x = simpleeval.DISALLOW_METHODS
+        simpleeval.DISALLOW_METHODS = []
         self.t('"{}:{}".format(1, 2)', '1:2')
+        simpleeval.DISALLOW_METHODS = x
 
     def test_function_args_none(self):
         def foo():
@@ -366,6 +383,7 @@ class TestTryingToBreakOut(DRYTest):
         dis = simpleeval.DISALLOW_PREFIXES
         simpleeval.DISALLOW_PREFIXES = ['func_']
 
+        self.t('houdini.trapdoor()', 42)
         self.t('houdini._quasi_private()', 84)
 
         # and return things to normal
@@ -375,9 +393,15 @@ class TestTryingToBreakOut(DRYTest):
     def test_builtins_private_access(self):
         # explicit attempt of the exploit from perkinslr
         with self.assertRaises(simpleeval.FeatureNotAvailable):
-            self.t("True.__class__.__class__.__base__.__subclasses__()[-1].__init__.func_globals['sys'].exit(1)", 42)
+            self.t("True.__class__.__class__.__base__.__subclasses__()[-1]"
+                   ".__init__.func_globals['sys'].exit(1)", 42)
 
 
+    def test_string_format(self):
+        # python has so many ways to break out!
+        with self.assertRaises(simpleeval.FeatureNotAvailable):
+             self.t('"{string.__class__}".format(string="things")', 0)
+
 class TestCompoundTypes(DRYTest):
     """ Test the compound-types edition of the library """
 
@@ -518,6 +542,9 @@ class TestNames(DRYTest):
         self.s.names['c'] = {'i': 11}
 
         self.t("c['i']", 11)
+        self.t("c.get('i')", 11)
+        self.t("c.get('j', 11)", 11)
+        self.t("c.get('j')", None)
 
         # you still can't assign though:
 
@@ -641,6 +668,29 @@ class TestSimpleEval(unittest.TestCase):
         self.assertEqual(simple_eval('randint(200) < 200 and rand() > 0'), True)
 
 
+class TestMethodChaining(unittest.TestCase):
+    def test_chaining_correct(self):
+        """
+            Contributed by Khalid Grandi (xaled).
+        """
+        class A(object):
+            def __init__(self):
+                self.a = "0"
+
+            def add(self, b):
+                self.a += "-add" + str(b)
+                return self
+
+            def sub(self, b):
+                self.a += "-sub" + str(b)
+                return self
+
+            def tostring(self):
+                return str(self.a)
+
+        x = A()
+        self.assertEqual(simple_eval("x.add(1).sub(2).sub(3).tostring()", names={"x": x}), "0-add1-sub2-sub3")
+
 class TestExtendingClass(unittest.TestCase):
     """
         It should be pretty easy to extend / inherit from the SimpleEval class,
@@ -659,10 +709,45 @@ class TestExtendingClass(unittest.TestCase):
 
         self.assertEqual(e.eval('"stuff happens"'), "stuff happens")
         self.assertEqual(e.eval('22 + 20'), 42)
+        self.assertEqual(e.eval('int("42")'), 42)
 
         with self.assertRaises(simpleeval.FeatureNotAvailable):
             e.eval('"  blah  ".strip()')
 
 
-if __name__ == '__main__':
+class TestExceptions(unittest.TestCase):
+    """
+        confirm a few attributes exist properly and haven't been
+        eaten by 2to3 or whatever... (see #41)
+    """
+
+    def test_functionnotdefined(self):
+        try:
+            raise FunctionNotDefined("foo", "foo in bar")
+        except FunctionNotDefined as e:
+            assert hasattr(e, 'func_name')
+            assert getattr(e, 'func_name') == 'foo'
+            assert hasattr(e, 'expression')
+            assert getattr(e, 'expression') == 'foo in bar'
+
+    def test_namenotdefined(self):
+        try:
+            raise NameNotDefined("foo", "foo in bar")
+        except NameNotDefined as e:
+            assert hasattr(e, 'name')
+            assert getattr(e, 'name') == 'foo'
+            assert hasattr(e, 'expression')
+            assert getattr(e, 'expression') == 'foo in bar'
+
+
+    def test_attributedoesnotexist(self):
+        try:
+            raise AttributeDoesNotExist("foo", "foo in bar")
+        except AttributeDoesNotExist as e:
+            assert hasattr(e, 'attr')
+            assert getattr(e, 'attr') == 'foo'
+            assert hasattr(e, 'expression')
+            assert getattr(e, 'expression') == 'foo in bar'
+
+if __name__ == '__main__':  # pragma: no cover
     unittest.main()
